#!/usr/bin/python

class FilterModule(object):
    '''Various utilities for manipulating NetBox data'''
    def __init__(self):
        self.virtual_iface_types = ('bridge', 'lag', 'virtual')

    def filters(self):
        return {
            'iface_real': self.iface_real,
            'iface_peer': self.iface_peer,
            'iface_vlans': self.iface_vlans
        }

    def iface_real(self, interfaces):
        '''Return only non-virtual interfaces'''
        for iface in interfaces:
            if iface.get('type', {}).get('value') not in self.virtual_iface_types:
                yield iface

    def iface_peer(self, interfaces):
        '''Return the name of the device connected to this interface'''
        for iface in interfaces:
            endpoint = (iface.get('connected_endpoints') or [{}])[0]
            if device := endpoint.get('device', {}).get('name'):
                yield device
            else:
                yield None

    def iface_vlans(self, interfaces):
        '''Returns a list of allowed VLANs for each interface'''
        for iface in interfaces:
            match iface.get('mode', {}).get('value'):
                case 'access':
                    yield [iface.get('untagged_vlan')] if 'untagged_vlan' in iface else []
                case 'tagged':
                    yield iface.get('tagged_vlans', [])
                case _:
                    yield []
